function Trec12Predict()
dataName = './data/UFdatasets/Trec12.mat';
fileName = './results/real/Trec12predict';


load(dataName);
A = full(Problem.A);
A(A>20)=0;
A = A';
N = A;
N(N==0)=nan;

% Should we generate data?
genData = 0;

NINSTANCES = 5;
k = 25;

% Which algorithms to run.
Cap = 1;
NMF = 1;
WNMF = 1;
SVD = 1;

%% Generate data.
if genData == 1
    O = A;   % Original matrix.
    save(fileName, 'O', '-append');
    nonzer = find(O);
    for instance = 1 : NINSTANCES
        % new holdout set
        H{instance} = nonzer(rand(numel(nonzer), 1) < 0.1);
    end
    save(fileName, 'H', '-append');
end


%% Run algorithms.
load(dataName);
load(fileName);
for instance = 1 : NINSTANCES
    holdOutIdx = H{instance};
    A = O;
    N = O;
    A(holdOutIdx) = 0;
    N(holdOutIdx) = nan;
    N(N==0) = nan;  % Turn zeros to nans;
    
    
    if Cap == 1
        [Bcap{instance}, Ccap{instance}] = Capricorn(N, k, 'MAX_CYCLES', 2);
        save(fileName, 'Bcap', 'Ccap', '-append');
    end
    
    
    if NMF == 1
        [Bnmf{instance}, Cnmf{instance}] = nmfParkWrapper(A, k);
        save(fileName, 'Bnmf', 'Cnmf', '-append');
    end
    
    
    if WNMF == 1
        [Bwnmf{instance}, Cwnmf{instance}] = wnmfrule(N, k);
        save(fileName, 'Bwnmf', 'Cwnmf', '-append');
    end
    
    
    if SVD == 1
        [Bsvd{instance}, Csvd{instance}] = svdWrapper(A, k);
        save(fileName, 'Bsvd', 'Csvd', '-append');
    end
end


%% Print results.
load(fileName);
for instance = 1 : NINSTANCES
    Acap = MaxProduct(Bcap{instance}, Ccap{instance});
    resultCap(instance) = nnz(round(Acap(H{instance})) == O(H{instance})) / numel(O(H{instance}));
    
    Anmf = Bnmf{instance} * Cnmf{instance};
    resultNmf(instance) = nnz(round(Anmf(H{instance})) == O(H{instance})) / numel(O(H{instance}));
    
    Awnmf = Bwnmf{instance} * Cwnmf{instance};
    resultWnmf(instance) = nnz(round(Awnmf(H{instance})) == O(H{instance})) / numel(O(H{instance}));
    
    Asvd = Bsvd{instance} * Csvd{instance};
    resultSvd(instance) = nnz(round(Asvd(H{instance})) == O(H{instance})) / numel(O(H{instance}));
end

fprintf('Capricorn:  %d%%\n', 100 * mean(resultCap));
fprintf('NMF:  %d%%\n', 100 * mean(resultNmf));
fprintf('WNMF:  %d%%\n', 100 * mean(resultWnmf));
fprintf('SVD:  %d%%\n', 100 * mean(resultSvd));


%% Print only results where something is claimed.
for instance = 1 : NINSTANCES
    resultCap(instance) = nnz(round(Acap(H{instance})) == O(H{instance}) & Acap(H{instance}) ~= 0) ...
        / nnz(Acap(H{instance}) ~= 0);
    
    resultNmf(instance) = nnz(round(Anmf(H{instance})) == O(H{instance}) & Anmf(H{instance}) ~= 0) ...
        / nnz(Anmf(H{instance}) ~= 0);
    
    resultWnmf(instance) = nnz(round(Awnmf(H{instance})) == O(H{instance}) & Awnmf(H{instance}) ~= 0) ...
        / nnz(Awnmf(H{instance}) ~= 0);
    
    resultSvd(instance) = nnz(round(Asvd(H{instance})) == O(H{instance}) & Asvd(H{instance}) ~= 0) ...
        / nnz(Asvd(H{instance}) ~= 0);
end

fprintf('\n\nPrediction results for elements where methods output nonzero.\n')
fprintf('Capricorn:  %d%%\n', 100 * mean(resultCap));
fprintf('NMF:  %d%%\n', 100 * mean(resultNmf));
fprintf('WNMF:  %d%%\n', 100 * mean(resultWnmf));
fprintf('SVD:  %d%%\n', 100 * mean(resultSvd));

end
